@@ -7,7 +7,8 @@ import datetime
 import io
 import pathlib
 import random
-from typing import Any, Iterable
+import traceback
+from typing import Any, Iterable, Sequence
 
 import discord
 import flanautils
@@ -180,12 +181,12 @@ class DiscordBot(MultiBot[Bot]):
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def _get_replied_message(self, original_message: constants.DISCORD_MESSAGE) -> Message | None:
         try:
-            replied_discord_message = original_message.reference.resolved
+            replied_original_message = original_message.reference.resolved
         except AttributeError:
             return
 
-        if not isinstance(replied_discord_message, discord.DeletedReferencedMessage):
-            return await self._get_message(replied_discord_message)
+        if not isinstance(replied_original_message, discord.DeletedReferencedMessage):
+            return await self._get_message(replied_original_message)
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def _get_text(self, original_message: constants.DISCORD_MESSAGE) -> str:
@@ -196,7 +197,7 @@ class DiscordBot(MultiBot[Bot]):
         try:
             await user.original_object.edit(mute=True)
         except discord.errors.HTTPException:
-            raise UserDisconnectedError
+            raise UserDisconnectedError(user.name)
 
     @staticmethod
     def _parse_html_to_discord_markdown(text: str | None) -> str | None:
@@ -250,7 +251,7 @@ class DiscordBot(MultiBot[Bot]):
         try:
             await user.original_object.edit(mute=False)
         except discord.errors.HTTPException:
-            raise UserDisconnectedError
+            raise UserDisconnectedError(user.name)
 
     # ---------------------------------------------- #
     #                    HANDLERS                    #
@@ -289,7 +290,7 @@ class DiscordBot(MultiBot[Bot]):
             user.save(('roles',))
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
-    async def clear(self, n_messages: int, chat: int | str | Chat | Message):  # todo2 test
+    async def clear(self, n_messages: int, chat: int | str | Chat | Message):
         if n_messages > 100:
             raise LimitError('El máximo es 100.')
 
@@ -315,24 +316,40 @@ class DiscordBot(MultiBot[Bot]):
                 database_message_to_delete.save()
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
-    async def delete_message(self, message_to_delete: int | str | Message, chat: int | str | Chat | Message = None):  # todo2 test
-        chat = await self.get_chat(chat)
-        match message_to_delete:
-            case int() | str():
-                message_to_delete = self.Message.find_one({'platform': self.platform.value, 'id': str(message_to_delete), 'chat': chat.object_id})
-            case self.Message() if message_to_delete.original_object and message_to_delete.chat and message_to_delete.chat == chat:
-                chat = None
-
-        if chat and chat.original_object:  # todo3 los 3 delete_message son casi identicos y la estructura de match ifs es enrevesada
-            await (await chat.original_object.fetch_message(message_to_delete.id)).delete()
-        elif message_to_delete.original_object:
-            await message_to_delete.original_object.delete()
+    async def delete_message(
+        self,
+        message_to_delete: int | str | Message,
+        chat: int | str | Chat | Message = None,
+        raise_not_found=False
+    ):
+        if isinstance(message_to_delete, self.Message) and message_to_delete.original_object:
+            original_message = message_to_delete.original_object
         else:
-            raise ValueError('The original discord object of the message or chat is needed')
+            chat = await self.get_chat(chat)
+            chat.pull_from_database()
+            if not isinstance(message_to_delete, Message):
+                message_to_delete = self.Message.find_one({'platform': self.platform.value, 'id': int(message_to_delete), 'chat': chat.object_id})
+            original_message = await chat.original_object.fetch_message(message_to_delete.id)
+
+        try:
+            await original_message.delete()
+        except discord.errors.NotFound:
+            if raise_not_found:
+                raise NotFoundError(traceback.format_exc().splitlines()[-1])
+        except discord.errors.Forbidden:
+            return
 
         message_to_delete.is_deleted = True
         message_to_delete.save()
 
+    # noinspection PyTypeChecker
+    def distribute_buttons(self, texts: Sequence[str], vertically=False) -> list[list[str]]:
+        texts = [f'{text[:constants.DISCORD_BUTTON_MAX_CHARACTERS - 3]}...' if len(text) > constants.DISCORD_BUTTON_MAX_CHARACTERS else text for text in texts]
+
+        if len(texts) <= constants.DISCORD_BUTTONS_MAX and vertically:
+            return flanautils.chunks(texts, 1)
+        return flanautils.chunks(texts, constants.DISCORD_BUTTONS_MAX)
+
     async def find_audit_entries(
         self,
         group_: int | str | Chat | Message,
@@ -478,7 +495,7 @@ class DiscordBot(MultiBot[Bot]):
         try:
             return user.original_object.voice.deaf
         except AttributeError:
-            raise UserDisconnectedError
+            raise UserDisconnectedError(user.name)
 
     async def is_muted(self, user: int | str | User, group_: int | str | Chat | Message) -> bool:
         user = await self.get_user(user, group_)
@@ -498,14 +515,14 @@ class DiscordBot(MultiBot[Bot]):
         try:
             return user.original_object.voice.self_deaf
         except AttributeError:
-            raise UserDisconnectedError
+            raise UserDisconnectedError(user.name)
 
     async def is_self_muted(self, user: int | str | User, group_: int | str | Chat | Message) -> bool:
         user = await self.get_user(user, group_)
         try:
             return user.original_object.voice.self_mute
         except AttributeError:
-            raise UserDisconnectedError
+            raise UserDisconnectedError(user.name)
 
     async def remove_role(self, user: int | str | User, group_: int | str | Chat | Message, role: int | str | Role):
         user = await self.get_user(user, group_)
@@ -534,7 +551,7 @@ class DiscordBot(MultiBot[Bot]):
         *,
         buttons_key: Any = None,
         reply_to: int | str | Message = None,
-        contents: dict = None,
+        data: dict = None,
         silent: bool = False,
         send_as_file: bool = None,
         edit=False
@@ -574,15 +591,19 @@ class DiscordBot(MultiBot[Bot]):
             if buttons_key is not None:
                 message.buttons_info.key = buttons_key
 
-            message.original_object = await message.original_object.edit(**kwargs)
-            last_contents = message.contents
-            if media_content := getattr(media, 'content', None):
-                del last_contents['media']
-            message.contents = {'media': media_content}
-            if contents is None:
-                message.contents |= last_contents
+            try:
+                message.original_object = await message.original_object.edit(**kwargs)
+            except discord.errors.NotFound:
+                return
+
+            if data is None:
+                if media is not None:
+                    message.data['media'] = media.content
             else:
-                message.contents |= contents
+                if media is None:
+                    message.data = data
+                else:
+                    message.data = {'media': media.content} | data
 
             message.update_last_edit()
             message.save()
@@ -606,9 +627,10 @@ class DiscordBot(MultiBot[Bot]):
             raise
 
         bot_message.buttons_info = ButtonsInfo(buttons=buttons, key=buttons_key)
-        bot_message.contents = {'media': getattr(media, 'content', None)}
-        if contents is not None:
-            bot_message.contents |= contents
+        if media:
+            bot_message.data['media'] = media.content
+        if data:
+            bot_message.data |= data
 
         bot_message.save()
 
