@@ -19,17 +19,17 @@ import functools
 import random
 import traceback
 from abc import ABC, abstractmethod
-from typing import Any, Callable, Generic, Iterable, Type, TypeVar, overload
+from typing import Any, Callable, Generic, Iterable, Sequence, Type, TypeVar, overload
 
 import discord
 import flanautils
 import telethon.events
 import telethon.events.common
-from flanautils import AmbiguityError, Media, NotFoundError, OrderedSet, RatioMatch, return_if_first_empty, shift_args_if_called
+from flanautils import AmbiguityError, Media, NotFoundError, OrderedSet, ScoreMatch, return_if_first_empty, shift_args_if_called
 
 from multibot import constants
 from multibot.exceptions import BadRoleError, LimitError, SendError, UserDisconnectedError
-from multibot.models import Ban, Button, Chat, Message, Mute, Penalty, Platform, RegisteredButtonCallback, RegisteredCallback, RegisteredCallbackBase, Role, User
+from multibot.models import Ban, Button, Chat, Message, Mute, Penalty, Platform, RegisteredButtonCallback, RegisteredCallback, Role, User
 
 
 # ---------------------------------------------------------- #
@@ -40,16 +40,33 @@ def find_message(func_: Callable = None, /, return_if_not_found=False) -> Callab
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         async def wrapper(self: MultiBot, *args, **kwargs):
-            args = (*args, *kwargs.values())
-            if not (message := flanautils.find(args, Message)):
-                if event := flanautils.find(args, constants.MESSAGE_EVENT):
+            def take_arg(type_: Type, args_, kwargs_):
+                object_ = None
+                new_args = []
+                for arg in args_:
+                    if isinstance(arg, type_):
+                        object_ = arg
+                    else:
+                        new_args.append(arg)
+                new_kwargs = {}
+                for k, v in kwargs_.items():
+                    if isinstance(v, type_):
+                        object_ = v
+                    else:
+                        new_kwargs[k] = v
+                return object_, new_args, new_kwargs
+
+            message, args, kwargs = take_arg(Message, args, kwargs)
+            if not message:
+                event, args, kwargs = take_arg(constants.MESSAGE_EVENT, args, kwargs)
+                if event:
                     message = await self._get_message(event)
                 elif return_if_not_found:
                     return
                 else:
                     raise NotFoundError('No message object')
 
-            return await func(self, message)
+            return await func(self, message, *args, **kwargs)
 
         return wrapper
 
@@ -61,10 +78,10 @@ def admin(func_: Callable = None, /, is_=True, send_negative=False) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message):
+        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
             message = message
             if is_ is message.author.is_admin or message.chat.is_private:
-                return await func(self, message)
+                return await func(self, message, *args, **kwargs)
             await self.accept_button_event(message)
             if send_negative:
                 await self.send_negative(message)
@@ -89,9 +106,9 @@ def bot_mentioned(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message):
+        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
             if is_ is self.is_bot_mentioned(message):
-                return await func(self, message)
+                return await func(self, message, *args, **kwargs)
             await self.accept_button_event(message)
 
         return wrapper
@@ -104,9 +121,9 @@ def group(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message):
+        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
             if is_ is message.chat.is_group:
-                return await func(self, message)
+                return await func(self, message, *args, **kwargs)
             await self.accept_button_event(message)
 
         return wrapper
@@ -117,9 +134,9 @@ def group(func_: Callable = None, /, is_=True) -> Callable:
 def ignore_self_message(func: Callable) -> Callable:
     @functools.wraps(func)
     @find_message
-    async def wrapper(self: MultiBot, message: Message):
+    async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
         if message.author.id != self.id:
-            return await func(self, message)
+            return await func(self, message, *args, **kwargs)
         await self.accept_button_event(message)
 
     return wrapper
@@ -130,9 +147,9 @@ def inline(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message):
+        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
             if message.is_inline is None or is_ is message.is_inline:
-                return await func(self, message)
+                return await func(self, message, *args, **kwargs)
             await self.accept_button_event(message)
 
         return wrapper
@@ -156,21 +173,21 @@ def parse_arguments(func: Callable) -> Callable:
     async def wrapper(*args, **kwargs) -> Any:
         def parse_buttons(buttons_) -> list[list[Button]] | None:
             match buttons_:
-                case [str(), *_] as buttons_:
+                case [*_, str()] as buttons_:
                     buttons_ = [[Button(button_text, False) for button_text in buttons_]]
-                case [(str(), bool()), *_] as buttons_:
+                case [*_, (str(), bool())] as buttons_:
                     buttons_ = [[Button(button_text, is_checked) for button_text, is_checked in buttons_]]
-                case [Button(), *_] as buttons_:
+                case [*_, Button()] as buttons_:
                     buttons_ = [list(buttons_)]
-                case [[str(), *_], *_] as buttons_:
+                case [*_, [*_, str()]] as buttons_:
                     buttons_ = list(buttons_)
                     for i, buttons_row in enumerate(buttons_):
                         buttons_[i] = [Button(button_text, False) for button_text in buttons_row]
-                case [[(str(), bool()), *_], *_] as buttons_:
+                case [*_, [*_, (str(), bool())]] as buttons_:
                     buttons_ = list(buttons_)
                     for i, buttons_row in enumerate(buttons_):
                         buttons_[i] = [Button(button_text, is_checked) for button_text, is_checked in buttons_row]
-                case [[Button(), *_], *_] as buttons_:
+                case [*_, [*_, Button()]] as buttons_:
                     buttons_ = list(buttons_)
                     for i, buttons_row in enumerate(buttons_):
                         buttons_[i] = list(buttons_row)
@@ -206,6 +223,7 @@ def parse_arguments(func: Callable) -> Callable:
                 case _:
                     buttons = parse_buttons(arg)
 
+        message = kwargs.get('message', message)
         chat = await self.get_chat(kwargs.get('chat', chat))
         if 'buttons' in kwargs:
             buttons = parse_buttons(kwargs['buttons'])
@@ -243,9 +261,9 @@ def reply(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message):
+        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
             if is_ is bool(message.replied_message):
-                return await func(self, message)
+                return await func(self, message, *args, **kwargs)
             await self.accept_button_event(message)
 
         return wrapper
@@ -286,18 +304,6 @@ class MultiBot(Generic[T], ABC):
     async def _ban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
 
-    async def _call_registered_callback(self, callback: RegisteredCallbackBase, message: Message):
-        try:
-            await callback(message)
-        except Exception:
-            if constants.SEND_EXCEPTION_MESSAGE_LINES:
-                await self.send(f'{random.choice(constants.EXCEPTION_PHRASES)}\n'
-                                '\n'
-                                '...\n'
-                                '\n'.join(traceback.format_exc().splitlines()[-constants.SEND_EXCEPTION_MESSAGE_LINES:]),
-                                message)
-            raise
-
     async def _check_penalties(self, penalty_class: Type[Penalty], unpenalize_method: Callable):
         penalties = penalty_class.find({'platform': self.platform.value}, lazy=True)
 
@@ -348,7 +354,11 @@ class MultiBot(Generic[T], ABC):
         pass
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
-    async def _get_message(self, event: constants.MESSAGE_EVENT) -> Message:
+    async def _get_message(
+        self,
+        event: constants.MESSAGE_EVENT,
+        pull_overwrite_fields: Iterable[str] = ('_id',)
+    ) -> Message:
         original_message = event if isinstance(event, constants.ORIGINAL_MESSAGE) else await self._get_original_message(event)
 
         message = self.Message(
@@ -368,7 +378,7 @@ class MultiBot(Generic[T], ABC):
         if message.buttons_info:
             message.buttons_info.pressed_text = await self._get_button_pressed_text(event)
             message.buttons_info.presser_user = await self._get_button_presser_user(event)
-        message.save(pull_overwrite_fields=('_id', 'config'))
+        message.save(pull_overwrite_fields=pull_overwrite_fields)
 
         return message
 
@@ -407,11 +417,16 @@ class MultiBot(Generic[T], ABC):
                 await self.send_error(str(e), context)
             except (SendError, NotFoundError) as e:
                 await self.send_error(str(e), context)
-            except UserDisconnectedError:
-                await self.send_error('El usuario no está conectado.', context)
+            except UserDisconnectedError as e:
+                await self.send_error(f'{e} no está conectado.', context)
             except AmbiguityError:
                 if constants.RAISE_AMBIGUITY_ERROR:
                     await self.send_error(f'Hay varias acciones relacionadas con tu mensaje. ¿Puedes especificar un poco más? {random.choice(constants.SAD_EMOJIS)}', context)
+            except Exception:
+                if constants.SEND_EXCEPTION_MESSAGE_LINES:
+                    traceback_message = '\n'.join(traceback.format_exc().splitlines()[-constants.SEND_EXCEPTION_MESSAGE_LINES:])
+                    await self.send(f'{random.choice(constants.EXCEPTION_PHRASES)}\n\n...\n{traceback_message}', context)
+                raise
 
     async def _mute(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
@@ -420,25 +435,19 @@ class MultiBot(Generic[T], ABC):
     def _parse_callbacks(
         text: str,
         registered_callbacks: list[RegisteredCallback],
-        ratio_reward_exponent: float = constants.RATIO_REWARD_EXPONENT,
+        score_reward_exponent: float = constants.SCORE_REWARD_EXPONENT,
         keywords_lenght_penalty: float = constants.KEYWORDS_LENGHT_PENALTY,
-        minimum_ratio_to_match: float = constants.MINIMUM_RATIO_TO_MATCH
+        minimum_score_to_match: float = constants.MINIMUM_SCORE_TO_MATCH
     ) -> OrderedSet[RegisteredCallback]:
-        text = text.lower()
-        original_text_words = OrderedSet()
+        text = flanautils.remove_accents(text.lower())
+
+        original_words = OrderedSet()
         for word in text.split():
-            if len(word) > constants.MAX_WORD_LENGTH:
-                continue
+            if len(word) <= constants.MAX_WORD_LENGTH:
+                original_words.add(word)
+        important_words = original_words - flanautils.CommonWords.get()
 
-            word = flanautils.replace(word, {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u',
-                                             'à': 'a', 'è': 'e', 'ì': 'i', 'ò': 'o', 'ù': 'u',
-                                             'â': 'a', 'ê': 'e', 'î': 'i', 'ô': 'o', 'û': 'u',
-                                             'ä': 'a', 'ë': 'e', 'ï': 'i', 'ö': 'o', 'ü': 'u'})
-            word = flanautils.translate(word, {'?': ' ', '¿': ' ', '!': ' ', '¡': ' '})
-            original_text_words.add(word)
-        text_words = original_text_words - flanautils.CommonWords.get()
-
-        matched_callbacks: set[tuple[int, RatioMatch[RegisteredCallback]]] = set()
+        matched_callbacks: set[tuple[int, ScoreMatch[RegisteredCallback]]] = set()
         always_callbacks: set[RegisteredCallback] = set()
         default_callbacks: set[RegisteredCallback] = set()
         for registered_callback in registered_callbacks:
@@ -448,36 +457,36 @@ class MultiBot(Generic[T], ABC):
                 default_callbacks.add(registered_callback)
             else:
                 mached_keywords_groups = 0
-                total_ratio = 0
+                total_score = 0
                 for keywords_group in registered_callback.keywords:
-                    text_words += [original_text_word for original_text_word in original_text_words if flanautils.cartesian_product_string_matching(original_text_word, keywords_group, min_ratio=registered_callback.min_ratio)]
-                    word_matches = flanautils.cartesian_product_string_matching(text_words, keywords_group, min_ratio=registered_callback.min_ratio)
-                    ratio = sum((max(matches.values()) + 1) ** ratio_reward_exponent for matches in word_matches.values())
+                    important_words |= {original_text_word for original_text_word in original_words if flanautils.cartesian_product_string_matching(original_text_word, keywords_group, min_score=registered_callback.min_score)}
+                    word_matches = flanautils.cartesian_product_string_matching(important_words, keywords_group, min_score=registered_callback.min_score)
+                    score = sum((max(matches.values()) + 1) ** score_reward_exponent for matches in word_matches.values())
                     try:
-                        ratio /= max(1., keywords_lenght_penalty * len(keywords_group))
+                        score /= max(1., keywords_lenght_penalty * len(keywords_group))
                     except ZeroDivisionError:
                         continue
-                    if ratio:
-                        total_ratio += ratio
+                    if score:
+                        total_score += score
                         mached_keywords_groups += 1
 
                 if mached_keywords_groups and mached_keywords_groups == len(registered_callback.keywords):
                     for priority, matched_callback in matched_callbacks:  # If the callback has been matched before but with less score it is overwritten, otherwise it is added
                         if matched_callback.element.callback == registered_callback.callback:
-                            if total_ratio > matched_callback.ratio:
+                            if total_score > matched_callback.score:
                                 matched_callbacks.discard((priority, matched_callback))
-                                matched_callbacks.add((registered_callback.priority, RatioMatch(registered_callback, total_ratio)))
+                                matched_callbacks.add((registered_callback.priority, ScoreMatch(registered_callback, total_score)))
                             break
                     else:
-                        matched_callbacks.add((registered_callback.priority, RatioMatch(registered_callback, total_ratio)))
+                        matched_callbacks.add((registered_callback.priority, ScoreMatch(registered_callback, total_score)))
 
         sorted_matched_callbacks = sorted(matched_callbacks, key=lambda e: e[1])
         sorted_matched_callbacks = sorted(sorted_matched_callbacks, key=lambda e: e[0], reverse=True)
         match sorted_matched_callbacks:
             case [(_priority, single)]:
                 determined_callbacks = always_callbacks | {single.element}
-            case [(first_priority, first), (second_priority, second), *_] if first.ratio >= minimum_ratio_to_match:
-                if first_priority == second_priority and first.ratio == second.ratio:
+            case [(first_priority, first), (second_priority, second), *_] if first.score >= minimum_score_to_match:
+                if first_priority == second_priority and first.score == second.score:
                     raise AmbiguityError(f'\n{first.element.callback}\n{second.element.callback}')
                 determined_callbacks = always_callbacks | {first.element}
             case _:
@@ -518,7 +527,10 @@ class MultiBot(Generic[T], ABC):
 
         for registered_callback in self._registered_button_callbacks:
             if registered_callback.key == message.buttons_info.key:
-                await self._call_registered_callback(registered_callback, message)
+                try:
+                    await registered_callback(message)
+                except Exception as e:
+                    await self._manage_exceptions(e, message)
 
     @ignore_self_message
     async def _on_new_message_raw(self, message: Message):
@@ -528,12 +540,15 @@ class MultiBot(Generic[T], ABC):
             await self._manage_exceptions(e, message)
         else:
             for registered_callback in registered_callbacks:
-                await self._call_registered_callback(registered_callback, message)
+                try:
+                    await registered_callback(message)
+                except Exception as e:
+                    await self._manage_exceptions(e, message)
 
     async def _on_ready(self):
         flanautils.init_db()
         print(f'{self.name} activado en {self.platform.name} (id: {self.id})')
-        await flanautils.do_every(constants.CHECK_MESSAGE_EVERY_SECONDS, self.clear_old_database_items)
+        await flanautils.do_every(constants.CLEAR_OLD_DATABASE_ITEMS_EVERY_SECONDS, self.clear_old_database_items)
         await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, self.check_bans)
         await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, self.check_mutes)
 
@@ -569,13 +584,40 @@ class MultiBot(Generic[T], ABC):
         cls.Message.collection.delete_many({'date': {'$lte': before_date}})
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
-    async def delete_message(self, message_to_delete: int | str | Message, chat: int | str | Chat | Message = None):
+    async def delete_message(
+        self,
+        message_to_delete: int | str | Message,
+        chat: int | str | Chat | Message = None,
+        raise_not_found=False
+    ):
+        pass
+
+    def distribute_buttons(self, texts: Sequence[str], vertically=False) -> list[list[str]]:
         pass
 
     @parse_arguments
     async def edit(self, *args, **kwargs) -> Message:
-        kwargs |= {'edit': True}
-        return await self.send(*args, **kwargs)
+        return await self.send(*args, **kwargs | {'edit': True})
+
+    async def filter_mention_ids(self, text: str | Iterable[str], message: Message, delete_names=False) -> list[str]:
+        if isinstance(text, str):
+            words = text.split()
+        else:
+            words = text
+
+        ids = []
+        if delete_names:
+            for user in message.mentions:
+                ids.append(user.name.lower())
+                ids.append(user.name.split('#')[0].lower())
+                ids.append(str(user.id))
+        else:
+            for user in message.mentions:
+                ids.append(str(user.id))
+        for role in await self.get_group_roles(message):
+            ids.append(str(role.id))
+
+        return [word for word in words if flanautils.remove_symbols(word).strip() not in ids]
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def find_role(self, role: int | str | Role, group_: int | str | Chat | Message) -> Role | None:
@@ -591,7 +633,7 @@ class MultiBot(Generic[T], ABC):
                 return flanautils.find(roles, condition=lambda role_: role_.name == role_name)
 
     async def find_users_by_roles(self, roles: Iterable[int | str | Role], group_: int | str | Chat | Message) -> list[User]:
-        pass
+        return []
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def get_chat(self, chat: int | str | User | Chat | Message) -> Chat | None:
@@ -638,9 +680,9 @@ class MultiBot(Generic[T], ABC):
     async def get_message(self, chat: int | str | User | Chat | Message, message: int | str | Message) -> Message | None:
         pass
 
-    @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
+    @return_if_first_empty([], exclude_self_types='MultiBot', globals_=globals())
     async def get_group_roles(self, group_: int | str | Chat | Message) -> list[Role]:
-        pass
+        return []
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def get_user(self, user: int | str | User, group_: int | str | Chat | Message = None) -> User | None:
@@ -707,17 +749,17 @@ class MultiBot(Generic[T], ABC):
             await self._unpenalize_later(mute, self._unmute, message)
 
     @overload
-    def register(self, func_: Callable = None, keywords=(), priority: int | float = 1, min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register(self, func_: Callable = None, keywords=(), priority: int | float = 1, min_score=constants.PARSE_CALLBACKS_MIN_SCORE_DEFAULT, always=False, default=False):
         pass
 
     @overload
-    def register(self, keywords=(), priority: int | float = 1, min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register(self, keywords=(), priority: int | float = 1, min_score=constants.PARSE_CALLBACKS_MIN_SCORE_DEFAULT, always=False, default=False):
         pass
 
     @shift_args_if_called(exclude_self_types='MultiBot', globals_=globals())
-    def register(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), priority: int | float = 1, min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), priority: int | float = 1, min_score=constants.PARSE_CALLBACKS_MIN_SCORE_DEFAULT, always=False, default=False):
         def decorator(func):
-            self._registered_callbacks.append(RegisteredCallback(func, keywords, priority, min_ratio, always, default))
+            self._registered_callbacks.append(RegisteredCallback(func, keywords, priority, min_score, always, default))
             return func
 
         return decorator(func_) if func_ else decorator
@@ -747,13 +789,13 @@ class MultiBot(Generic[T], ABC):
         self,
         text='',
         media: Media = None,
-        buttons: list[str | tuple[str, bool] | list[str | tuple[str, bool]]] | None = None,
+        buttons: list[str | tuple[str, bool] | Button | list[str | tuple[str, bool] | Button]] | None = None,
         chat: int | str | User | Chat | Message | None = None,
         message: Message = None,
         *,
         buttons_key: Any = None,
         reply_to: int | str | Message = None,
-        contents: dict = None,
+        data: dict = None,
         silent: bool = False,
         send_as_file: bool = None,
         edit=False
