@@ -6,7 +6,7 @@ import asyncio
 import functools
 import io
 import pathlib
-from typing import Any, Callable
+from typing import Any, Callable, Sequence
 
 import flanautils
 import pymongo
@@ -78,18 +78,24 @@ class TelegramBot(MultiBot[TelegramClient]):
         self.client.add_event_handler(self._on_new_message_raw, telethon.events.NewMessage)
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
-    async def _create_bot_message_from_telegram_bot_message(self, original_message: constants.TELEGRAM_MESSAGE, media: Media, chat: Chat, buttons: list[list[Button]] = None, buttons_key: Any = None, contents: dict = None) -> Message | None:
+    async def _create_bot_message_from_telegram_bot_message(
+        self,
+        original_message: constants.TELEGRAM_MESSAGE,
+        media: Media,
+        chat: Chat,
+        buttons: list[list[Button]] = None,
+        buttons_key: Any = None,
+        data: dict = None
+    ) -> Message | None:
         original_message._sender = await self.client.get_entity(self.id)
         original_message._chat = chat.original_object
         bot_message = await self._get_message(original_message)
         bot_message.buttons_info = ButtonsInfo(buttons=buttons, key=buttons_key)
         if media and media.bytes_ and len(media.bytes_) <= constants.PYMONGO_MEDIA_MAX_BYTES:
-            media = media.content
-        else:
-            media = None
-        bot_message.contents = {'media': media}
-        if contents is not None:
-            bot_message.contents |= contents
+            bot_message.data['media'] = media.content
+        if data:
+            bot_message.data |= data
+
         bot_message.save()
 
         return bot_message
@@ -325,24 +331,31 @@ class TelegramBot(MultiBot[TelegramClient]):
                 database_message_to_delete.save()
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
-    async def delete_message(self, message_to_delete: int | str | Message, chat: int | str | Chat | Message = None):
-        chat = await self.get_chat(chat)
-        match message_to_delete:
-            case int() | str():
-                message_to_delete = self.Message.find_one({'platform': self.platform.value, 'id': str(message_to_delete), 'chat': chat.object_id})
-            case self.Message() if message_to_delete.original_object and message_to_delete.chat and message_to_delete.chat == chat:
-                chat = None
-
-        if chat and chat.original_object:
-            await self.client.delete_messages(chat.original_object, message_to_delete.id)
-        elif message_to_delete.original_object:
+    async def delete_message(
+        self,
+        message_to_delete: int | str | Message,
+        chat: int | str | Chat | Message = None,
+        raise_not_found=False
+    ):
+        if isinstance(message_to_delete, self.Message) and message_to_delete.original_object:
             await message_to_delete.original_object.delete()
         else:
-            raise ValueError('The original telegram object of the message or chat is needed')
+            chat = await self.get_chat(chat)
+            chat.pull_from_database()
+            if not isinstance(message_to_delete, Message):
+                message_to_delete = self.Message.find_one({'platform': self.platform.value, 'id': int(message_to_delete), 'chat': chat.object_id})
+            await self.client.delete_messages(chat.original_object, message_to_delete.id)
 
         message_to_delete.is_deleted = True
         message_to_delete.save()
 
+    # noinspection PyTypeChecker
+    def distribute_buttons(self, texts: Sequence[str], vertically=False) -> list[list[str]]:
+        if vertically:
+            return flanautils.chunks(texts, 1)
+        else:
+            return flanautils.chunks(texts, constants.TELEGRAM_BUTTONS_MAX_PER_LINE)
+
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def get_chat(self, chat: int | str | User | Chat | Message) -> Chat | None:
         match chat:
@@ -400,7 +413,7 @@ class TelegramBot(MultiBot[TelegramClient]):
         *,
         buttons_key: Any = None,
         reply_to: int | str | Message = None,
-        contents: dict = None,
+        data: dict = None,
         silent: bool = False,
         send_as_file: bool = None,
         edit=False,
@@ -423,15 +436,15 @@ class TelegramBot(MultiBot[TelegramClient]):
 
         if message:
             if send_as_file is None:
-                word_matches = flanautils.cartesian_product_string_matching(message.text, constants.KEYWORDS['send_as_file'], min_ratio=constants.TELEGRAM_SEND_AS_FILE_RATIO_MIN_RATIO)
-                send_as_file_ratio = sum(max(matches.values()) for text_word, matches in word_matches.items())
-                kwargs['force_document'] = bool(send_as_file_ratio)
+                word_matches = flanautils.cartesian_product_string_matching(message.text, constants.KEYWORDS['send_as_file'], min_score=constants.TELEGRAM_SEND_AS_FILE_MIN_SCORE)
+                send_as_file_score = sum(max(matches.values()) for text_word, matches in word_matches.items())
+                kwargs['force_document'] = bool(send_as_file_score)
 
             if message.is_inline:
                 if media:
-                    if 'inline_media' not in message.contents:
-                        message.contents['inline_media'] = []
-                    message.contents['inline_media'].append(media)
+                    if 'inline_media' not in message.data:
+                        message.data['inline_media'] = []
+                    message.data['inline_media'].append(media)
                 return
             elif edit:
                 if buttons is not None:
@@ -444,23 +457,23 @@ class TelegramBot(MultiBot[TelegramClient]):
                     message.original_object = await message.original_object.edit(text, **kwargs)
                 except (
                         telethon.errors.rpcerrorlist.PeerIdInvalidError,
-                        telethon.errors.rpcerrorlist.UserIsBlockedError,
-                        telethon.errors.rpcerrorlist.MessageNotModifiedError
+                        telethon.errors.rpcerrorlist.MessageIdInvalidError,
+                        telethon.errors.rpcerrorlist.MessageNotModifiedError,
+                        telethon.errors.rpcerrorlist.UserIsBlockedError
                 ):
                     return
-                else:
-                    last_contents = message.contents
-                    if media_content := getattr(media, 'content', None):
-                        del last_contents['media']
-                    message.contents = {'media': media_content}
-                    if contents is None:
-                        message.contents |= last_contents
-                    else:
-                        message.contents |= contents
-                    message.update_last_edit()
-                    message.save()
 
-                    return message
+                if data is None:
+                    if media is not None:
+                        message.data['media'] = media.content
+                else:
+                    if media is None:
+                        message.data = data
+                    else:
+                        message.data = {'media': media.content} | data
+                message.update_last_edit()
+                message.save()
+                return message
 
         match reply_to:
             case str():
@@ -489,7 +502,7 @@ class TelegramBot(MultiBot[TelegramClient]):
                 else:
                     break
 
-        return await self._create_bot_message_from_telegram_bot_message(original_message, media, chat, buttons, buttons_key, contents)
+        return await self._create_bot_message_from_telegram_bot_message(original_message, media, chat, buttons, buttons_key, data)
 
     @inline
     async def send_inline_results(self, message: Message):
@@ -504,9 +517,9 @@ class TelegramBot(MultiBot[TelegramClient]):
         with flanautils.suppress_stderr():
             try:
                 try:
-                    await message.original_event.answer([await create_result(media) for media in message.contents['inline_media']])
+                    await message.original_event.answer([await create_result(media) for media in message.data['inline_media']])
                 except telethon.errors.rpcerrorlist.WebpageCurlFailedError:
-                    await message.original_event.answer([await create_result(media, prefer_bytes=True) for media in message.contents['inline_media']])
+                    await message.original_event.answer([await create_result(media, prefer_bytes=True) for media in message.data['inline_media']])
             except (KeyError, telethon.errors.rpcerrorlist.QueryIdInvalidError):
                 pass
 
